/* 
* -cr_beg-
* ********************************************************************
* IBM Confidential
* OCO Source Materials
* 5724-U38, 5724-U39, 5724-U48, 5655-V17, 5655-V19, 5655-V35, 5655-V37
* Copyright IBM Corp. 2013
* The source code for this program is not published or otherwise
* divested of its trade secrets, irrespective of what has
* been deposited with the U.S. Copyright Office.
* ********************************************************************
* -cr_end-
*/ 
#include "ioqoptimmaskorafunc.h"

char* OptimMaskChar1(   OCIExtProcContext *pCtx,
                        char *pArg1,        //1st Data argument
                        short sArg1Ind,
                        int iArg1Len,
                        int iArg1Cid,
                        int iArg1Cform,
                        char *pParamString, //provider-syntax string
                        short sPSInd,
                        int iPSLen,
                        int iPSCid,
                        int iPSCform,
                        short *pRetInd,
                        int *pRetLen)
{
   RETVAL retVal = ODPPFAILURE;
   char *pRetStr = NULL;

   //Declare elements enough for input arguments, here the input are 2
   //Character Argument (pArg1),
   //Parameter String (pParamString)
   COracleData *pDataInArr[2];

   //Declare a pointer for output
   COracleData *pDataOutArr;

   //Allocate the input data classes
   //This is a string class becuase the input is a char
   *(pDataInArr + 0) = (COracleData*) new COracleString(pArg1, sArg1Ind, iArg1Len, iArg1Cid, iArg1Cform);
   
   //This is a string class becuase the parameter string is always a char
   *(pDataInArr + 1) = (COracleData*) new COracleString(pParamString, sPSInd, iPSLen, iPSCid, iPSCform);
   
   //Allocate the output data class, this is a string because we are 
   //returning a char
   pDataOutArr = (COracleData*) new COracleString(NULL, 0, 0, 0, 0);

   //Call the OptimMaskUDF function to mask the input data
   retVal = OptimMaskUDF(pCtx, pDataInArr, 2, pDataOutArr, 1);
   if(ODPP_UDF_ERR_ORA_NULL_VALUE == retVal)
   {
      *pRetInd = OCI_IND_NULL;
      *pRetLen = 0;
      //==========================================
      //Before returning free what you allocated
      //==========================================
      delete *(pDataInArr + 0);
      delete *(pDataInArr + 1);
      delete pDataOutArr;
      return NULL;
   }
   
   //Get the output data
   pDataOutArr->GetString(&pRetStr, pRetInd, pRetLen, NULL, NULL);
   
   //==========================================
   //Before returning free what you allocated
   //==========================================
   delete *(pDataInArr + 0);
   delete *(pDataInArr + 1);
   delete pDataOutArr;

   return pRetStr;

}

char* OptimMaskStr2( OCIExtProcContext *pCtx,
                     char *pArg1,        //1st Data argument
                     short sArg1Ind,
                     int iArg1Len,
                     int iArg1Cid,
                     int iArg1Cform,
                     char *pArg2,        //2nd Data argument
                     short sArg2Ind,
                     int iArg2Len,
                     int iArg2Cid,
                     int iArg2Cform,
                     char *pParamString, //provider-syntax string
                     short sPSInd,
                     int iPSLen,
                     int iPSCid,
                     int iPSCform,
                     short *pRetInd,
                     int *pRetLen)
{
   RETVAL retVal = ODPPFAILURE;
   char *pRetStr = NULL;

   //Declare elements enough for input arguments, here the input are 3
   //Character Argument (pArg1),
   //Character Argument (pArg2),
   //Parameter String (pParamString)
   COracleData *pDataInArr[3];

   //Declare a pointer for output
   COracleData *pDataOutArr;

   //Allocate the input data classes
   //This is a string class becuase the input is a char
   *(pDataInArr + 0) = (COracleData*) new COracleString(pArg1, sArg1Ind, iArg1Len, iArg1Cid, iArg1Cform);

   //This is a string class becuase the input is a char
   *(pDataInArr + 1) = (COracleData*) new COracleString(pArg2, sArg2Ind, iArg2Len, iArg2Cid, iArg2Cform);

   //This is a string class becuase the parameter string is always a char
   *(pDataInArr + 2) = (COracleData*) new COracleString(pParamString, sPSInd, iPSLen, iPSCid, iPSCform);
   
   //Allocate the output data class, this is a string because we are 
   //returning a char
   pDataOutArr = (COracleData*) new COracleString(NULL, 0, 0, 0, 0);

   //Call the OptimMaskUDF function to mask the input data
   retVal = OptimMaskUDF(pCtx, pDataInArr, 3, pDataOutArr, 1);
   if(ODPP_UDF_ERR_ORA_NULL_VALUE == retVal)
   {
      *pRetInd = OCI_IND_NULL;
      *pRetLen = 0;
      //==========================================
      //Before returning free what you allocated
      //==========================================
      delete *(pDataInArr + 0);
      delete *(pDataInArr + 1);
      delete *(pDataInArr + 2);
      delete pDataOutArr;
      return NULL;
   }
   
   //Get the output data
   pDataOutArr->GetString(&pRetStr, pRetInd, pRetLen, NULL, NULL);
   
   //==========================================
   //Before returning free what you allocated
   //==========================================
   delete *(pDataInArr + 0);
   delete *(pDataInArr + 1);
   delete *(pDataInArr + 2);
   delete pDataOutArr;

   return pRetStr;

}

char* OptimMaskStr3( OCIExtProcContext *pCtx,
                     char *pArg1,        //1st Data argument
                     short sArg1Ind,
                     int iArg1Len,
                     int iArg1Cid,
                     int iArg1Cform,
                     char *pArg2,        //2nd Data argument
                     short sArg2Ind,
                     int iArg2Len,
                     int iArg2Cid,
                     int iArg2Cform,
                     char *pArg3,        //3rd Data argument
                     short sArg3Ind,
                     int iArg3Len,
                     int iArg3Cid,
                     int iArg3Cform,
                     char *pParamString, //provider-syntax string
                     short sPSInd,
                     int iPSLen,
                     int iPSCid,
                     int iPSCform,
                     short *pRetInd,
                     int *pRetLen)
{
   RETVAL retVal = ODPPFAILURE;
   char *pRetStr = NULL;

   //Declare elements enough for input arguments, here the input are 4
   //Character Argument (pArg1),
   //Character Argument (pArg2),
   //Character Argument (pArg3),
   //Parameter String   (pParamString)
   COracleData *pDataInArr[4];

   //Declare a pointer for output
   COracleData *pDataOutArr;

   //Allocate the input data classes
   //This is a string class becuase the input is a char
   *(pDataInArr + 0) = (COracleData*) new COracleString(pArg1, sArg1Ind, iArg1Len, iArg1Cid, iArg1Cform);

   //This is a string class becuase the input is a char
   *(pDataInArr + 1) = (COracleData*) new COracleString(pArg2, sArg2Ind, iArg2Len, iArg2Cid, iArg2Cform);

   //This is a string class becuase the input is a char
   *(pDataInArr + 2) = (COracleData*) new COracleString(pArg3, sArg3Ind, iArg3Len, iArg3Cid, iArg3Cform);

   //This is a string class becuase the parameter string is always a char
   *(pDataInArr + 3) = (COracleData*) new COracleString(pParamString, sPSInd, iPSLen, iPSCid, iPSCform);
   
   //Allocate the output data class, this is a string because we are 
   //returning a char
   pDataOutArr = (COracleData*) new COracleString(NULL, 0, 0, 0, 0);

   //Call the OptimMaskUDF function to mask the input data
   retVal = OptimMaskUDF(pCtx, pDataInArr, 4, pDataOutArr, 1);
   if(ODPP_UDF_ERR_ORA_NULL_VALUE == retVal)
   {
      *pRetInd = OCI_IND_NULL;
      *pRetLen = 0;
      //==========================================
      //Before returning free what you allocated
      //==========================================
      delete *(pDataInArr + 0);
      delete *(pDataInArr + 1);
      delete *(pDataInArr + 2);
      delete *(pDataInArr + 3);
      delete pDataOutArr;
      return NULL;
   }
   
   //Get the output data
   pDataOutArr->GetString(&pRetStr, pRetInd, pRetLen, NULL, NULL);
   
   //==========================================
   //Before returning free what you allocated
   //==========================================
   delete *(pDataInArr + 0);
   delete *(pDataInArr + 1);
   delete *(pDataInArr + 2);
   delete *(pDataInArr + 3);
   delete pDataOutArr;

   return pRetStr;

}

char* OptimMaskStr4( OCIExtProcContext *pCtx,
                     char *pArg1,        //1st Data argument
                     short sArg1Ind,
                     int iArg1Len,
                     int iArg1Cid,
                     int iArg1Cform,
                     char *pArg2,        //2nd Data argument
                     short sArg2Ind,
                     int iArg2Len,
                     int iArg2Cid,
                     int iArg2Cform,
                     char *pArg3,        //3rd Data argument
                     short sArg3Ind,
                     int iArg3Len,
                     int iArg3Cid,
                     int iArg3Cform,
                     char *pArg4,        //4th Data argument
                     short sArg4Ind,
                     int iArg4Len,
                     int iArg4Cid,
                     int iArg4Cform,
                     char *pParamString, //provider-syntax string
                     short sPSInd,
                     int iPSLen,
                     int iPSCid,
                     int iPSCform,
                     short *pRetInd,
                     int *pRetLen)
{
   RETVAL retVal = ODPPFAILURE;
   char *pRetStr = NULL;

   //Declare elements enough for input arguments, here the input are 5
   //Character Argument (pArg1),
   //Character Argument (pArg2),
   //Character Argument (pArg3),
   //Character Argument (pArg4),
   //Parameter String   (pParamString)
   COracleData *pDataInArr[5];

   //Declare a pointer for output
   COracleData *pDataOutArr;

   //Allocate the input data classes
   //This is a string class becuase the input is a char
   *(pDataInArr + 0) = (COracleData*) new COracleString(pArg1, sArg1Ind, iArg1Len, iArg1Cid, iArg1Cform);

   //This is a string class becuase the input is a char
   *(pDataInArr + 1) = (COracleData*) new COracleString(pArg2, sArg2Ind, iArg2Len, iArg2Cid, iArg2Cform);

   //This is a string class becuase the input is a char
   *(pDataInArr + 2) = (COracleData*) new COracleString(pArg3, sArg3Ind, iArg3Len, iArg3Cid, iArg3Cform);

   //This is a string class becuase the input is a char
   *(pDataInArr + 3) = (COracleData*) new COracleString(pArg4, sArg4Ind, iArg4Len, iArg4Cid, iArg4Cform);

   //This is a string class becuase the parameter string is always a char
   *(pDataInArr + 4) = (COracleData*) new COracleString(pParamString, sPSInd, iPSLen, iPSCid, iPSCform);
   
   //Allocate the output data class, this is a string because we are 
   //returning a char
   pDataOutArr = (COracleData*) new COracleString(NULL, 0, 0, 0, 0);

   //Call the OptimMaskUDF function to mask the input data
   retVal = OptimMaskUDF(pCtx, pDataInArr, 5, pDataOutArr, 1);
   if(ODPP_UDF_ERR_ORA_NULL_VALUE == retVal)
   {
      *pRetInd = OCI_IND_NULL;
      *pRetLen = 0;
      //==========================================
      //Before returning free what you allocated
      //==========================================
      delete *(pDataInArr + 0);
      delete *(pDataInArr + 1);
      delete *(pDataInArr + 2);
      delete *(pDataInArr + 3);
      delete *(pDataInArr + 4);
      delete pDataOutArr;
      return NULL;
   }
   
   //Get the output data
   pDataOutArr->GetString(&pRetStr, pRetInd, pRetLen, NULL, NULL);
   
   //==========================================
   //Before returning free what you allocated
   //==========================================
   delete *(pDataInArr + 0);
   delete *(pDataInArr + 1);
   delete *(pDataInArr + 2);
   delete *(pDataInArr + 3);
   delete *(pDataInArr + 4);
   delete pDataOutArr;

   return pRetStr;

}

char* OptimMaskStr5( OCIExtProcContext *pCtx,
                     char *pArg1,        //1st Data argument
                     short sArg1Ind,
                     int iArg1Len,
                     int iArg1Cid,
                     int iArg1Cform,
                     char *pArg2,        //2nd Data argument
                     short sArg2Ind,
                     int iArg2Len,
                     int iArg2Cid,
                     int iArg2Cform,
                     char *pArg3,        //3rd Data argument
                     short sArg3Ind,
                     int iArg3Len,
                     int iArg3Cid,
                     int iArg3Cform,
                     char *pArg4,        //4th Data argument
                     short sArg4Ind,
                     int iArg4Len,
                     int iArg4Cid,
                     int iArg4Cform,
                     char *pArg5,        //5th Data argument
                     short sArg5Ind,
                     int iArg5Len,
                     int iArg5Cid,
                     int iArg5Cform,
                     char *pParamString, //provider-syntax string
                     short sPSInd,
                     int iPSLen,
                     int iPSCid,
                     int iPSCform,
                     short *pRetInd,
                     int *pRetLen)
{
   RETVAL retVal = ODPPFAILURE;
   char *pRetStr = NULL;

   //Declare elements enough for input arguments, here the input are 6
   //Character Argument (pArg1),
   //Character Argument (pArg2),
   //Character Argument (pArg3),
   //Character Argument (pArg4),
   //Character Argument (pArg5),
   //Parameter String   (pParamString)
   COracleData *pDataInArr[6];

   //Declare a pointer for output
   COracleData *pDataOutArr;

   //Allocate the input data classes
   //This is a string class becuase the input is a char
   *(pDataInArr + 0) = (COracleData*) new COracleString(pArg1, sArg1Ind, iArg1Len, iArg1Cid, iArg1Cform);

   //This is a string class becuase the input is a char
   *(pDataInArr + 1) = (COracleData*) new COracleString(pArg2, sArg2Ind, iArg2Len, iArg2Cid, iArg2Cform);

   //This is a string class becuase the input is a char
   *(pDataInArr + 2) = (COracleData*) new COracleString(pArg3, sArg3Ind, iArg3Len, iArg3Cid, iArg3Cform);

   //This is a string class becuase the input is a char
   *(pDataInArr + 3) = (COracleData*) new COracleString(pArg4, sArg4Ind, iArg4Len, iArg4Cid, iArg4Cform);

   //This is a string class becuase the input is a char
   *(pDataInArr + 4) = (COracleData*) new COracleString(pArg5, sArg5Ind, iArg5Len, iArg5Cid, iArg5Cform);

   //This is a string class becuase the parameter string is always a char
   *(pDataInArr + 5) = (COracleData*) new COracleString(pParamString, sPSInd, iPSLen, iPSCid, iPSCform);
   
   //Allocate the output data class, this is a string because we are 
   //returning a char
   pDataOutArr = (COracleData*) new COracleString(NULL, 0, 0, 0, 0);

   //Call the OptimMaskUDF function to mask the input data
   retVal = OptimMaskUDF(pCtx, pDataInArr, 6, pDataOutArr, 1);
   if(ODPP_UDF_ERR_ORA_NULL_VALUE == retVal)
   {
      *pRetInd = OCI_IND_NULL;
      *pRetLen = 0;
      //==========================================
      //Before returning free what you allocated
      //==========================================
      delete *(pDataInArr + 0);
      delete *(pDataInArr + 1);
      delete *(pDataInArr + 2);
      delete *(pDataInArr + 3);
      delete *(pDataInArr + 4);
      delete *(pDataInArr + 5);
      delete pDataOutArr;
      return NULL;
   }
   
   //Get the output data
   pDataOutArr->GetString(&pRetStr, pRetInd, pRetLen, NULL, NULL);
   
   //==========================================
   //Before returning free what you allocated
   //==========================================
   delete *(pDataInArr + 0);
   delete *(pDataInArr + 1);
   delete *(pDataInArr + 2);
   delete *(pDataInArr + 3);
   delete *(pDataInArr + 4);
   delete *(pDataInArr + 5);
   delete pDataOutArr;

   return pRetStr;
}

char* OptimMaskStr6( OCIExtProcContext *pCtx,
                     char *pArg1,        //1st Data argument
                     short sArg1Ind,
                     int iArg1Len,
                     int iArg1Cid,
                     int iArg1Cform,
                     char *pArg2,        //2nd Data argument
                     short sArg2Ind,
                     int iArg2Len,
                     int iArg2Cid,
                     int iArg2Cform,
                     char *pArg3,        //3rd Data argument
                     short sArg3Ind,
                     int iArg3Len,
                     int iArg3Cid,
                     int iArg3Cform,
                     char *pArg4,        //4th Data argument
                     short sArg4Ind,
                     int iArg4Len,
                     int iArg4Cid,
                     int iArg4Cform,
                     char *pArg5,        //5th Data argument
                     short sArg5Ind,
                     int iArg5Len,
                     int iArg5Cid,
                     int iArg5Cform,
                     char *pArg6,        //6th Data argument
                     short sArg6Ind,
                     int iArg6Len,
                     int iArg6Cid,
                     int iArg6Cform,
                     char *pParamString, //provider-syntax string
                     short sPSInd,
                     int iPSLen,
                     int iPSCid,
                     int iPSCform,
                     short *pRetInd,
                     int *pRetLen)
{
   RETVAL retVal = ODPPFAILURE;
   char *pRetStr = NULL;

   //Declare elements enough for input arguments, here the input are 7
   //Character Argument (pArg1),
   //Character Argument (pArg2),
   //Character Argument (pArg3),
   //Character Argument (pArg4),
   //Character Argument (pArg5),
   //Character Argument (pArg6),
   //Parameter String   (pParamString)
   COracleData *pDataInArr[7];

   //Declare a pointer for output
   COracleData *pDataOutArr;

   //Allocate the input data classes
   //This is a string class becuase the input is a char
   *(pDataInArr + 0) = (COracleData*) new COracleString(pArg1, sArg1Ind, iArg1Len, iArg1Cid, iArg1Cform);

   //This is a string class becuase the input is a char
   *(pDataInArr + 1) = (COracleData*) new COracleString(pArg2, sArg2Ind, iArg2Len, iArg2Cid, iArg2Cform);

   //This is a string class becuase the input is a char
   *(pDataInArr + 2) = (COracleData*) new COracleString(pArg3, sArg3Ind, iArg3Len, iArg3Cid, iArg3Cform);

   //This is a string class becuase the input is a char
   *(pDataInArr + 3) = (COracleData*) new COracleString(pArg4, sArg4Ind, iArg4Len, iArg4Cid, iArg4Cform);

   //This is a string class becuase the input is a char
   *(pDataInArr + 4) = (COracleData*) new COracleString(pArg5, sArg5Ind, iArg5Len, iArg5Cid, iArg5Cform);

   //This is a string class becuase the input is a char
   *(pDataInArr + 5) = (COracleData*) new COracleString(pArg6, sArg6Ind, iArg6Len, iArg6Cid, iArg6Cform);

   //This is a string class becuase the parameter string is always a char
   *(pDataInArr + 6) = (COracleData*) new COracleString(pParamString, sPSInd, iPSLen, iPSCid, iPSCform);
   
   //Allocate the output data class, this is a string because we are 
   //returning a char
   pDataOutArr = (COracleData*) new COracleString(NULL, 0, 0, 0, 0);

   //Call the OptimMaskUDF function to mask the input data
   retVal = OptimMaskUDF(pCtx, pDataInArr, 7, pDataOutArr, 1);
   if(ODPP_UDF_ERR_ORA_NULL_VALUE == retVal)
   {
      *pRetInd = OCI_IND_NULL;
      *pRetLen = 0;
      //==========================================
      //Before returning free what you allocated
      //==========================================
      delete *(pDataInArr + 0);
      delete *(pDataInArr + 1);
      delete *(pDataInArr + 2);
      delete *(pDataInArr + 3);
      delete *(pDataInArr + 4);
      delete *(pDataInArr + 5);
      delete *(pDataInArr + 6);
      delete pDataOutArr;
      return NULL;
   }
   
   //Get the output data
   pDataOutArr->GetString(&pRetStr, pRetInd, pRetLen, NULL, NULL);
   
   //==========================================
   //Before returning free what you allocated
   //==========================================
   delete *(pDataInArr + 0);
   delete *(pDataInArr + 1);
   delete *(pDataInArr + 2);
   delete *(pDataInArr + 3);
   delete *(pDataInArr + 4);
   delete *(pDataInArr + 5);
   delete *(pDataInArr + 6);
   delete pDataOutArr;

   return pRetStr;
}

OCINumber *OptimMaskCharNum(  OCIExtProcContext *pCtx,
                              char *pArg1,        //1st Data argument
                              short sArg1Ind,
                              int iArg1Len,
                              int iArg1Cid,
                              int iArg1Cform,
                              OCINumber *pArg2,   //2nd Data argument
                              short sArg2Ind,
                              char *pParamString, //provider-syntax string
                              short sPSInd,
                              int iPSLen,
                              int iPSCid,
                              int iPSCform,
                              short *pRetInd)
{
   RETVAL retVal = ODPPFAILURE;
   OCINumber *pRetNum = NULL;

   //Declare elements enough for input arguments, here the input are 3
   //Character Argument (pArg1),
   //OCINumber Argument (pArg2),
   //Parameter String (pParamString)
   COracleData *pDataInArr[3];

   //Declare a pointer for output
   COracleData *pDataOutArr;

   //Allocate the input data classes
   //This is a string class becuase the input is a char
   *(pDataInArr + 0) = (COracleData*) new COracleString(pArg1, sArg1Ind, iArg1Len, iArg1Cid, iArg1Cform);

   //This is a Number class becuase the input is a OCINumber
   *(pDataInArr + 1) = (COracleData*) new COracleNumber(pArg2, sArg2Ind);   

   //This is a string class becuase the parameter string is always a char
   *(pDataInArr + 2) = (COracleData*) new COracleString(pParamString, sPSInd, iPSLen, iPSCid, iPSCform);
   
   //Allocate the output data class, this is a number class because 
   //we are returning a OCINumber.
   pDataOutArr = (COracleData*) new COracleNumber(pRetNum,0);

   //Call the OptimMaskUDF function to mask the input data
   retVal = OptimMaskUDF(pCtx, pDataInArr, 3, pDataOutArr, 1);
   if(ODPP_UDF_ERR_ORA_NULL_VALUE == retVal)
   {
      *pRetInd = OCI_IND_NULL;
      //==========================================
      //Before returning free what you allocated
      //==========================================
      delete *(pDataInArr + 0);
      delete *(pDataInArr + 1);
      delete *(pDataInArr + 2);
      delete pDataOutArr;
      return NULL;
   }

   //Get the output data
   pDataOutArr->GetOciNumber(&pRetNum, pRetInd);

   //==========================================
   //Before returning free what you allocated
   //==========================================
   delete *(pDataInArr + 0);
   delete *(pDataInArr + 1);
   delete *(pDataInArr + 2);
   delete pDataOutArr;

   return pRetNum;

}

int *OptimMaskStr2Int( OCIExtProcContext *pCtx,
                        char *pArg1,        //1st Data argument
                        short sArg1Ind,
                        int iArg1Len,
                        int iArg1Cid,
                        int iArg1Cform,
                        char *pArg2,        //2nd Data argument
                        short sArg2Ind,
                        int iArg2Len,
                        int iArg2Cid,
                        int iArg2Cform,
                        char *pParamString, //provider-syntax string
                        short sPSInd,
                        int iPSLen,
                        int iPSCid,
                        int iPSCform,
                        short *pRetInd)
{
   RETVAL retVal = ODPPFAILURE;
   int *pRetInteger = NULL;

   //Declare elements enough for input arguments, here the input are 3
   //Character Argument (pArg1),
   //Character Argument (pArg2),
   //Parameter String   (pParamString)
   COracleData *pDataInArr[3];

   //Declare a pointer for output
   COracleData *pDataOutArr;

   //Allocate the input data classes
   //This is a string class becuase the input is a char
   *(pDataInArr + 0) = (COracleData*) new COracleString(pArg1, sArg1Ind, iArg1Len, iArg1Cid, iArg1Cform);

   //This is a string class becuase the input is a char
   *(pDataInArr + 1) = (COracleData*) new COracleString(pArg2, sArg2Ind, iArg2Len, iArg2Cid, iArg2Cform);

   //This is a string class becuase the parameter string is always a char
   *(pDataInArr + 2) = (COracleData*) new COracleString(pParamString, sPSInd, iPSLen, iPSCid, iPSCform);
   
   //Allocate the output data class, this is a number class because 
   //we are returning a OCINumber.
   pDataOutArr = (COracleData*) new COracleNumber(pRetInteger,0);

   //Call the OptimMaskUDF function to mask the input data
   retVal = OptimMaskUDF(pCtx, pDataInArr, 3, pDataOutArr, 1);
   if(ODPP_UDF_ERR_ORA_NULL_VALUE == retVal)
   {
      *pRetInd = OCI_IND_NULL;
      //==========================================
      //Before returning free what you allocated
      //==========================================
      delete *(pDataInArr + 0);
      delete *(pDataInArr + 1);
      delete *(pDataInArr + 2);
      delete pDataOutArr;
      return NULL;
   }

   //Get the output data
   pDataOutArr->GetInteger(&pRetInteger, pRetInd);

   //==========================================
   //Before returning free what you allocated
   //==========================================
   delete *(pDataInArr + 0);
   delete *(pDataInArr + 1);
   delete *(pDataInArr + 2);
   delete pDataOutArr;

   return pRetInteger;

}


OCIDateTime* OptimMaskTS1( OCIExtProcContext *pCtx,
                           OCIDateTime *pArg1,   //1st Data argument
                           short sArg1Ind,
                           char *pParamString, //provider-syntax string
                           short sPSInd,
                           int iPSLen,
                           int iPSCid,
                           int iPSCform,
                           short *pRetInd)
{
   RETVAL retVal = ODPPFAILURE;
   OCIDateTime *pRetDateTime = NULL;

   //Declare elements enough for input arguments, here the input are 2
   //Date Argument (pArg1),
   //Parameter String (pParamString)
   COracleData *pDataInArr[2];

   //Declare a pointer for output
   COracleData *pDataOutArr;

   //Allocate the input data classes
   //This is a string class becuase the input is a char
   *(pDataInArr + 0) = (COracleData*) new COracleDateTime(pArg1, sArg1Ind);
   
   //This is a string class becuase the parameter string is always a char
   *(pDataInArr + 1) = (COracleData*) new COracleString(pParamString, sPSInd, iPSLen, iPSCid, iPSCform);
   
   //Allocate the output data class, this is a date time because we are 
   //returning a date
   pDataOutArr = (COracleData*) new COracleDateTime(pRetDateTime, 0);

   //Call the OptimMaskUDF function to mask the input data
   retVal = OptimMaskUDF(pCtx, pDataInArr, 2, pDataOutArr, 1);
   if(ODPP_UDF_ERR_ORA_NULL_VALUE == retVal)
   {
      *pRetInd = OCI_IND_NULL;
      //==========================================
      //Before returning free what you allocated
      //==========================================
      delete *(pDataInArr + 0);
      delete *(pDataInArr + 1);
      delete pDataOutArr;
      return NULL;
   }
   
   //Get the output data
   pDataOutArr->GetDateTime(&pRetDateTime, pRetInd);
   
   //==========================================
   //Before returning free what you allocated
   //==========================================
   delete *(pDataInArr + 0);
   delete *(pDataInArr + 1);
   delete pDataOutArr;

   return pRetDateTime;
}


